@@ -146,6 +146,16 @@ class TestStyle(unittest.TestCase):
             style.doc.getvalue(),
             six.b(''))
 
+    def test_href_link(self):
+        style = ReSTStyle(ReSTDocument())
+        style.start_a(attrs=[('href', 'http://example.org')])
+        style.doc.write('example')
+        style.end_a()
+        self.assertEqual(
+            style.doc.getvalue(),
+            six.b('`example <http://example.org>`__ ')
+        )
+
     def test_escape_href_link(self):
         style = ReSTStyle(ReSTDocument())
         style.start_a(attrs=[('href', 'http://example.org')])
@@ -153,15 +163,14 @@ class TestStyle(unittest.TestCase):
         style.end_a()
         self.assertEqual(
             style.doc.getvalue(),
-            six.b('`foo\\: the next bar`_ \n\n.. _foo\\: the next '
-                  'bar: http://example.org\n'))
+            six.b('`foo\\: the next bar <http://example.org>`__ '))
 
     def test_handle_no_text_hrefs(self):
         style = ReSTStyle(ReSTDocument())
         style.start_a(attrs=[('href', 'http://example.org')])
         style.end_a()
         self.assertEqual(style.doc.getvalue(),
-                         six.b('`<http://example.org>`_ '))
+                         six.b('`<http://example.org>`__ '))
 
     def test_sphinx_reference_label_html(self):
         style = ReSTStyle(ReSTDocument())
@@ -279,3 +288,17 @@ class TestStyle(unittest.TestCase):
 
         self.assertEqual(style.doc.getvalue(),
                          six.b("\n\n\n* foo\n\n\n  \n  * bar\n  "))
+
+    def test_external_link(self):
+        style = ReSTStyle(ReSTDocument())
+        style.doc.target = 'html'
+        style.external_link('MyLink', 'http://example.com/foo')
+        self.assertEqual(style.doc.getvalue(),
+                         six.b('`MyLink <http://example.com/foo>`_'))
+
+
+    def test_external_link_in_man_page(self):
+        style = ReSTStyle(ReSTDocument())
+        style.doc.target = 'man'
+        style.external_link('MyLink', 'http://example.com/foo')
+        self.assertEqual(style.doc.getvalue(), six.b('MyLink'))
